import networkx as nx
import matplotlib.pyplot as plt
from matplotlib.widgets import Button

# Creation d'un graphe exemple : grille carree de cote 7
G = nx.grid_2d_graph(7, 7)
pos = {(i, j): (i, -j) for i, j in G.nodes()}  # Positionnement des sommets pour une meilleure visualisation

selected_nodes = set()
previous_selected_nodes = []

# Creer la figure et les axes
fig, ax = plt.subplots(figsize=(10, 8))

def draw_graph(highlight_neighbors=False):
    ax.clear()  # Clear the current axes
    
    nx.draw(G, pos, with_labels=False, node_color='lightblue', edge_color='gray', node_size=500, ax=ax)
    
    if highlight_neighbors:
        neighbors = set(selected_nodes)
        for node in selected_nodes:
            neighbors.update(G.neighbors(node))
        nx.draw_networkx_nodes(G, pos, nodelist=list(neighbors), node_color='orange', node_size=600, ax=ax)
    
    nx.draw_networkx_nodes(G, pos, nodelist=list(selected_nodes), node_color='red', node_size=600, ax=ax)
    plt.draw()  # Redraw the current figure

def on_click(event):
    if event.inaxes is not None and event.inaxes == ax:
        # Trouver le sommet le plus proche du clic
        min_dist = float('inf')
        closest_node = None
        for node, (x, y) in pos.items():
            dist = (event.xdata - x) ** 2 + (event.ydata - y) ** 2
            if dist < min_dist:
                min_dist = dist
                closest_node = node

        # Ajouter ou supprimer le sommet des sommets selectionnes
        if closest_node is not None:
            if closest_node in selected_nodes:
                selected_nodes.remove(closest_node)
            else:
                selected_nodes.add(closest_node)
        
        # Redessiner le graphe avec les nouveaux sommets selectionnes
        draw_graph()

def on_button_click(event):
    draw_graph(highlight_neighbors=True)

def on_next_step_click(event):
    global selected_nodes
    global previous_selected_nodes
    neighbors = set()
    for node in selected_nodes:
        neighbors.update(G.neighbors(node))
    selected_nodes.update(neighbors)
    previous_selected_nodes.append(set(selected_nodes))
    draw_graph(highlight_neighbors=False)

def on_calculation_click(event):
    all_nodes = set(G.nodes)
    steps = 0
    temp_selected_nodes = selected_nodes.copy()
    while temp_selected_nodes != all_nodes:
        neighbors = set()
        for node in temp_selected_nodes:
            neighbors.update(G.neighbors(node))
        temp_selected_nodes.update(neighbors)
        steps += 1
        if steps > len(G.nodes):
            print("Impossible de selectionner tous les sommets du graphe.")
            return
    print("Il faudra {} etapes pour selectionner tous les sommets du graphe.".format(steps))

def on_deselect_all_click(event):
    global selected_nodes
    selected_nodes.clear()
    draw_graph()

# Connecter l'evenement de clic sur la figure
fig.canvas.mpl_connect('button_press_event', on_click)

# Creer un bouton pour mettre a jour le voisinage
ax_button = plt.axes([0.1, 0.02, 0.15, 0.05])  # Position du bouton
btn = Button(ax_button, 'Afficher Voisins')
btn.on_clicked(on_button_click)

# Creer un bouton pour l'extension
ax_next_step_button = plt.axes([0.3, 0.02, 0.15, 0.05])  # Position du bouton de prochaine etape
next_step_btn = Button(ax_next_step_button, 'Etape suivante')
next_step_btn.on_clicked(on_next_step_click)

# Creer un bouton pour le calcul
ax_calculation_button = plt.axes([0.5, 0.02, 0.15, 0.05])  # Position du bouton de calcul
calculation_btn = Button(ax_calculation_button, 'Calcul')
calculation_btn.on_clicked(on_calculation_click)

# Creer un bouton pour tout deselectionner
ax_deselect_all_button = plt.axes([0.7, 0.02, 0.15, 0.05])  # Position du bouton de deselection
deselect_all_btn = Button(ax_deselect_all_button, 'Deselectionner tout')
deselect_all_btn.on_clicked(on_deselect_all_click)

# Dessiner le graphe initial
draw_graph()

plt.show()

